﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Volga_IT_17_Soldatkin
{
    public class Field
    {
        public int Width { get; set; }
        public int Height { get; set; }
        public List<Creature> Creatures { get; set; }
        public Player Player { get; set; }

        public int ForceMax
        {
            get
            {
                return Width + Height;
            }
        }

        public bool IsWin { get; set; }
        public bool IsDefeat { get; set; }

        public void InitEmpty(int w, int h)
        {
            Width = w;
            Height = h;
            Creatures = new List<Creature>();
            Player = new Player();
            Creatures.Add(Player);
            IsDefeat = false;
            IsWin = false;
        }

        public bool CanMovePlayerTo(int row, int column)
        {
            if (column < 0 || row < 0)
                return false;

            if (column >= Width || row >= Height)
                return false;

            if (Player.Row == row && Player.Column == column)
                return false;

            if (Player.Row != row && Player.Column != column)
                return false;

            int absDX = Math.Abs(Player.Row - row);
            int absDY = Math.Abs(Player.Column - column);

            if (absDX <= 1 && absDY <= 1)
            {
                var creature = GetCreatureAt(row, column);

                if (creature == null)
                    return true;

                if (creature.Force <= Player.Force)
                    return true;

                return false;
            }

            var distantCreature = GetCreatureAt(row, column);
            if (distantCreature == null)
                return false;

            if (distantCreature.Force != Player.Force)
                return false;

            var tiles = TilesOnThePlayerWay(row, column);
            var supostati = tiles.Select(tile => GetCreatureAt(tile.Key, tile.Value)).Where(vrajina => vrajina != null && vrajina != distantCreature && vrajina != Player);

            if (supostati.All(supostat => supostat.Force < Player.Force))
                return true;

            return false;
        }

        public IEnumerable<KeyValuePair<int, int>> TilesOnThePlayerWay(int destRow, int destColumn)
        {
            int stpX = destRow < Player.Row ? -1 : 1;
            for (int row = Player.Row; row != destRow; row += stpX)
                yield return new KeyValuePair<int, int>(row, Player.Column);

            int stpY = destColumn < Player.Column ? -1 : 1;
            for (int column = Player.Column; column != destColumn; column += stpY)
                yield return new KeyValuePair<int, int>(Player.Row, column);

            yield return new KeyValuePair<int, int>(destRow, destColumn);
        }

        public void TryMovePlayerTo(int row, int column)
        {
            if (IsDefeat || IsWin)
                return;

            if (!CanMovePlayerTo(row, column))
                return;

            var tileCreature = GetCreatureAt(row, column);
            if (tileCreature != null && tileCreature.Force == Player.Force)
                Player.Force++;

            var creatures = TilesOnThePlayerWay(row, column).Select(p => GetCreatureAt(p.Key, p.Value)).Where(c => c != null && c != Player);
            foreach (var c in creatures)
            {
                Creatures.Remove(c);
            }

            Player.Row = row;
            Player.Column = column;
           

            IsWin = Player.Force == ForceMax + 1;
            if (IsWin)
            {
                MainManager.Instance.RaiseFieldChanged();
                MainManager.Instance.RaiseBattleStateChanged();
                return;
            }

            bool enemyCreationSuccess = CreateRandimEnemyOnTurn();
            MainManager.Instance.RaiseFieldChanged();

            if (!enemyCreationSuccess || AllPositions.All(p => !CanMovePlayerTo(p.Key, p.Value)))
            {
                IsDefeat = true;
                MainManager.Instance.RaiseBattleStateChanged();
            }
        }

        public IEnumerable<KeyValuePair<int, int>> AllPositions
        {
            get
            {
                for (int row = 0; row < Height; row++)
                {
                    for (int column = 0; column < Width; column++)
                    {
                        yield return new KeyValuePair<int, int>(row, column);
                    }
                }
            }
        }

        public bool CreateRandimEnemyOnTurn()
        {
            int count = MainManager.Instance.Random.Next(1, 4);
            for (int i = 0; i < count; i++)
            {
                bool created = CreateRandomEnemy();
                if (i == 0 && !created)
                    return false;
            }

            MainManager.Instance.RaiseFieldChanged();
            return true;
        }

        public bool CreateRandomEnemy()
        {
            List<KeyValuePair<int, int>> available = AllPositions.Where(p => IsPositionFree(p.Key, p.Value)).ToList();

            if (available.Count < 1)
                return false;
            
            KeyValuePair<int, int> pos = available[MainManager.Instance.Random.Next(0, available.Count)];
            Enemy e = new Enemy();
            e.Row = pos.Key;
            e.Column = pos.Value;
            e.Force = MainManager.Instance.Random.Next(0, ForceMax + 1);
            //e.Force = MainManager.Instance.Random.Next(0, 5);
            Creatures.Add(e);

            return true;
        }

        public bool IsPositionFree(int row, int column)
        {
            return GetCreatureAt(row, column) == null;
        }

        public Creature GetCreatureAt(int row, int column)
        {
            return Creatures.FirstOrDefault(c => c.Row == row && c.Column == column);
        }
    }

    public class Enemy : Creature
    {
    }

    public class Player : Creature
    {
    }

    public class Creature
    {
        public int Force { get; set; }
        public int Row { get; set; }
        public int Column { get; set; }
    }
}
