﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Volga_IT_17_Soldatkin_Editor
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            MainManager.Instance.SelectionChanged += OnSelectionChanged;

            Render.Instance.Init(pictureBox1);
            NewMap();
        }

        private void OnSelectionChanged()
        {
            var c = MainManager.Instance.Field.GetCreatureAt(Render.Instance.SelectedRow, Render.Instance.SelectedColumn);

            if (c == null)
            {
                radioButton1Enemy.Checked = false;
                radioButton2Player.Checked = false;
                radioButton3Empty.Checked = true;

                textBoxForce.Text = string.Empty;
            }
            else
            {
                radioButton1Enemy.Checked = c is Enemy;
                radioButton2Player.Checked = c is Player;
                radioButton3Empty.Checked = false;

                textBoxForce.Text = c.Force.ToString();
            }
        }

        private void NewMap()
        {
            int w = 0;
            int h = 0;

            if (!int.TryParse(textBox1Width.Text, out w) || w < 1)
            {
                MessageBox.Show("Ширина должна быть положительным числом");
                return;
            }

            if (!int.TryParse(textBox2Height.Text, out h) || h < 1)
            {
                MessageBox.Show("Высота должна быть положительным числом");
                return;
            }

            Render.Instance.SelectedColumn = 0;
            Render.Instance.SelectedRow = 0;

            MainManager.Instance.Field = new Field();
            MainManager.Instance.Field.InitEmpty(w, h);
            MainManager.Instance.RaiseFieldChanged();
        }

        private void buttonNew_Click(object sender, EventArgs e)
        {
            NewMap();
        }

        private void buttonChangeTile_Click(object sender, EventArgs e)
        {
            var c = MainManager.Instance.Field.GetCreatureAt(Render.Instance.SelectedRow, Render.Instance.SelectedColumn);

            if (c != null)
            {
                MainManager.Instance.Field.Creatures.Remove(c);
                if (MainManager.Instance.Field.Player == c)
                    MainManager.Instance.Field.Player = null;
            }

            if (radioButton1Enemy.Checked)
            {
                Enemy enemy = new Enemy();
                enemy.Column = Render.Instance.SelectedColumn;
                enemy.Row = Render.Instance.SelectedRow;

                int force;
                if (!int.TryParse(textBoxForce.Text, out force))
                {
                    force = 0;
                    textBoxForce.Text = "0";
                }
                enemy.Force = force;
                MainManager.Instance.Field.Creatures.Add(enemy);
            }
            else if (radioButton2Player.Checked)
            {
                if (MainManager.Instance.Field.Player != null)
                {
                    MainManager.Instance.Field.Creatures.Remove(MainManager.Instance.Field.Player);
                    MainManager.Instance.Field.Player = null;
                }

                Player player = new Player();
                player.Column = Render.Instance.SelectedColumn;
                player.Row = Render.Instance.SelectedRow;

                int force;
                if (!int.TryParse(textBoxForce.Text, out force) || force < 0)
                {
                    force = 0;
                    textBoxForce.Text = "0";
                }
                player.Force = force;
                MainManager.Instance.Field.Creatures.Add(player);

                MainManager.Instance.Field.Player = player;
            }
            else if (radioButton3Empty.Checked)
            {

            }

            MainManager.Instance.RaiseFieldChanged();
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            if (MainManager.Instance.Field.Player == null)
            {
                MessageBox.Show("Не найден игрок");
                return;
            }

            saveFileDialog1.ShowDialog();
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            MainManager.Instance.Loader.Save(saveFileDialog1.FileName);
        }

        private void buttonSaveDifficulity_Click(object sender, EventArgs e)
        {
            Difficulity d = SerializeDifficulity();
            if (d == null)
                return;

            saveFileDialog2.ShowDialog();
        }

        private Difficulity SerializeDifficulity()
        {
            int w;
            int h;
            int enemyLimit;
            int enemyCount;

            if (!int.TryParse(textBoxDiffW.Text, out w) || w < 5 || w > 10)
            {
                MessageBox.Show("Ширина поля число от 5 до 10");
                return null;
            }

            if (!int.TryParse(textBoxDiffH.Text, out h) || h < 5 || h > 10)
            {
                MessageBox.Show("Высота поля число от 5 до 10");
                return null;
            }

            if (!int.TryParse(textBoxEnemyLimit.Text, out enemyLimit) || enemyLimit < 0)
            {
                MessageBox.Show("Лимит силы врагов число которое >= 0");
                return null;
            }

            if (!int.TryParse(textBoxMaxEnemy.Text, out enemyCount) || enemyCount < 1)
            {
                MessageBox.Show("Макс. число создавемых врагов должно быть > 0");
                return null;
            }

            Difficulity d = new Difficulity();
            d.SizeX = w;
            d.SizeY = h;
            d.EnemyForceLimit = enemyLimit;
            d.MaxGeneratingEnemy = enemyCount;
            return d;
        }

        private void saveFileDialog2_FileOk(object sender, CancelEventArgs e)
        {
            Difficulity d = SerializeDifficulity();
            if (d == null)
                return;

            File.WriteAllText(saveFileDialog2.FileName, d.Serialize(), Encoding.ASCII);
        }
    }
}
