﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Volga_IT_17_Soldatkin_Editor
{
    public class Render
    {
        public static Render Instance = new Render();

        private PictureBox box;
        private Graphics graphics;
        private int boxW;
        private int boxH;

        public int SelectedRow = 0;
        public int SelectedColumn = 0;

        public Render()
        {
            MainManager.Instance.FieldChanged += OnFieldChanged;
        }

        public void Init(PictureBox box)
        {
            this.box = box;
            boxW = box.Width;
            boxH = box.Height;

            Bitmap bitmap = new Bitmap(boxW, boxH);
            box.Image = bitmap;
            graphics = Graphics.FromImage(bitmap);

            box.MouseDown += OnBoxMouseDown;
        }

        private void OnBoxMouseDown(object sender, MouseEventArgs e)
        {
            int x = e.X;
            int y = e.Y;

            int row = (int)Math.Floor(y / RowH);
            int column = (int)Math.Floor(x / ColumnW);

            if (row < 0 || column < 0)
                return;

            if (column >= MainManager.Instance.Field.Width)
                return;

            if (row >= MainManager.Instance.Field.Height)
                return;

            SelectedColumn = column;
            SelectedRow = row;
            DrawState();

            MainManager.Instance.RaiseSelectionChanged();
        }

        private void OnFieldChanged()
        {
            DrawState();
        }

        private float ColumnW
        {
            get { return boxW / F.Width; }
        }

        private float RowH
        {
            get { return boxH / F.Height; }
        }

        private Field F
        {
            get { return MainManager.Instance.Field; }
        }

        public void DrawState()
        {
            if (graphics == null)
                return;

            graphics.Clear(Color.White);

            Brush b = new SolidBrush(Color.LightGreen);
            RectangleF rectF = new RectangleF(SelectedColumn * ColumnW, SelectedRow * RowH, ColumnW, RowH);
            graphics.FillRectangle(b, rectF);

            for (int i = 1; i < F.Width; i++)
            {
                float xPos = i * ColumnW;
                PointF p1 = new PointF(xPos, 0);
                PointF p2 = new PointF(xPos, boxH);

                graphics.DrawLine(Pens.Black, p1, p2);
            }

            for (int j = 1; j < F.Height; j++)
            {
                float yPos = j * RowH;
                PointF p1 = new PointF(0, yPos);
                PointF p2 = new PointF(boxW, yPos);
                graphics.DrawLine(Pens.Black, p1, p2);
            }

            foreach (var c in MainManager.Instance.Field.Creatures)
            {
                int row = c.Row;
                int col = c.Column;
                int force = c.Force;

                float textX = col * ColumnW;
                float textY = row * RowH;

                Player p = c as Player;
                if (p != null)
                {

                    Font font = new Font(FontFamily.GenericMonospace, 15, FontStyle.Bold);
                    Brush brush = new SolidBrush(Color.Blue);
                    graphics.DrawString("И" + force.ToString(), font, brush, textX, textY);

                    continue;
                }

                Enemy e = c as Enemy;
                if (e != null)
                {
                    Font font = new Font(FontFamily.GenericMonospace, 15, FontStyle.Bold);
                    Brush brush = new SolidBrush(Color.Red);
                    graphics.DrawString(force.ToString(), font, brush, textX, textY);

                    continue;
                }
            }

            box.Invalidate();
        }
    }
}
