﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Volga_IT_17_Soldatkin
{
    public class Loader
    {
        public void Save(string path)
        {
            StringBuilder b = new StringBuilder();

            b.Append(MainManager.Instance.Field.Width);
            b.Append(" ");
            b.Append(MainManager.Instance.Field.Height);
            b.Append(Environment.NewLine);

            b.Append(MainManager.Instance.Field.Player.Column);
            b.Append(" ");
            b.Append(MainManager.Instance.Field.Player.Row);
            b.Append(" ");
            b.Append(MainManager.Instance.Field.Player.Force);
            b.Append(" ");
            b.Append(MainManager.Instance.Field.Creatures.Count - 1);
            b.Append(Environment.NewLine);

            foreach (var c in MainManager.Instance.Field.Creatures)
            {
                if (c == MainManager.Instance.Field.Player)
                    continue;

                b.Append(c.Column);
                b.Append(" ");
                b.Append(c.Row);
                b.Append(" ");
                b.Append(c.Force);
                b.Append(Environment.NewLine);
            }

            File.WriteAllText(path, b.ToString(), Encoding.ASCII);
        }

        public void Load(string path)
        {
            if (!File.Exists(path))
            {
                MessageBox.Show("Файл не найден: " + path);
                return;
            }

            string[] content = File.ReadAllLines(path, Encoding.ASCII);

            if (content.Length < 2)
            {
                MessageBox.Show("В файле не может быть менее 2 строк");
                return;
            }

            string[] tokens1 = Split(content[0], ' ');

            if (tokens1.Length < 2)
            {
                MessageBox.Show("В первой строке должно быть 2 токена");
                return;
            }

            int w;
            int h;

            if (!int.TryParse(tokens1[0], out w) || !int.TryParse(tokens1[1], out h) || w < 0 || h < 0)
            {
                MessageBox.Show("Ошибка при разборе первой строки");
                return;
            }

            if (w < 5 || w > 10)
            {
                MessageBox.Show("Ширина должна быть от 5 до 10");
                return;
            }

            if (h < 5 || h > 10)
            {
                MessageBox.Show("Высота должна быть от 5 до 10");
                return;
            }

            string[] tokens2 = Split(content[1], ' ');

            if (tokens2.Length < 4)
            {
                MessageBox.Show("Во второй строке должно быть 4 токена");
                return;
            }

            int px;
            int py;
            int pForce;
            int enemyCount;

            if (!int.TryParse(tokens2[0], out px) || !int.TryParse(tokens2[1], out py) || !int.TryParse(tokens2[2], out pForce) || !int.TryParse(tokens2[3], out enemyCount))
            {
                MessageBox.Show("Ошибка при разборе строки 2");
                return;
            }

            if (px < 0 || px >= w)
            {
                MessageBox.Show("Позиция игрока по X невалидная");
                return;
            }

            if (py < 0 || py >= h)
            {
                MessageBox.Show("Позиция игрока по Y невалидная");
                return;
            }

            if (pForce < 0 || pForce >= w + h)
            {
                MessageBox.Show("Сила игрока невалидная");
                return;
            }

            Player loadedPlayer = new Player() { Column = px, Row = py, Force = pForce };
            List<Creature> creatures = new List<Creature>();
            creatures.Add(loadedPlayer);

            for (int line = 2; line < content.Length; line++)
            {
                if (string.Empty.Equals(content[line]))
                    continue;

                if (enemyCount == creatures.Count - 1)
                {
                    MessageBox.Show("Враг не будет загружен, так как их число уже превышено " + content[line]);
                    continue;
                }

                string[] eTokens = Split(content[line], ' ');

                if (eTokens.Length < 3)
                {
                    MessageBox.Show("В строках врагов должно быть 3 токена");
                    return;
                }

                int ex;
                int ey;
                int eForce;

                if (!int.TryParse(eTokens[0], out ex) || !int.TryParse(eTokens[1], out ey) || !int.TryParse(eTokens[2], out eForce))
                {
                    MessageBox.Show("Ошибка при разборе врага: " + content[line]);
                    return;
                }

                if (ex < 0 || ex >= w)
                {
                    MessageBox.Show("Позиция врага по X невалидная " + content[line]);
                    return;
                }

                if (ey < 0 || ey >= h)
                {
                    MessageBox.Show("Позиция врага по Y невалидная " + content[line]);
                    return;
                }

                if (eForce < 0 || eForce > w + h)
                {
                    MessageBox.Show("Сила врага невалидная " + content[line]);
                    return;
                }

                creatures.Add(new Enemy() { Column = ex, Row = ey, Force = eForce });
            }

            if (enemyCount != creatures.Count - 1)
            {
                MessageBox.Show("Врагов недосточно");
                return;
            }

            MainManager.Instance.Field.InitEmpty(w, h);
            MainManager.Instance.Field.Creatures = creatures;
            MainManager.Instance.Field.Player = loadedPlayer;
            MainManager.Instance.RaiseFieldChanged();
        }

        private string[] Split(string content, char c)
        {
            return content.Split(new[] { c }, StringSplitOptions.RemoveEmptyEntries);
        }
    }
}
