﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Volga_IT_17_Soldatkin
{
    public class Render
    {
        public static Render Instance = new Render();

        private PictureBox box;
        private Graphics graphics;
        private int boxW;
        private int boxH;
        private Image imageIvan;
        private Image[] enemyImages;
        private Image imageWin;
        private Image imageLoose;

        public Render()
        {
            MainManager.Instance.FieldChanged += OnFieldChanged;

            string ivanPath = Directory.GetCurrentDirectory();
            ivanPath = Path.Combine(ivanPath, "Images");
            ivanPath = Path.Combine(ivanPath, "ivan.jpg");
            imageIvan = Image.FromFile(ivanPath);

            enemyImages = new Image[3];
            for (int i = 1; i <= enemyImages.Length; i++)
            {
                string ePath = Directory.GetCurrentDirectory();
                ePath = Path.Combine(ePath, "Images");
                ePath = Path.Combine(ePath, "e" + i + ".jpg");
                enemyImages[i - 1] = Image.FromFile(ePath);
            }

            string winPath = Directory.GetCurrentDirectory();
            winPath = Path.Combine(winPath, "Images");
            winPath = Path.Combine(winPath, "victory.jpg");
            imageWin = Image.FromFile(winPath);

            string loosePath = Directory.GetCurrentDirectory();
            loosePath = Path.Combine(loosePath, "Images");
            loosePath = Path.Combine(loosePath, "loose.jpg");
            imageLoose = Image.FromFile(loosePath);
        }

        public void Init(PictureBox box)
        {
            this.box = box;
            boxW = box.Width;
            boxH = box.Height;

            Bitmap bitmap = new Bitmap(boxW, boxH);
            box.Image = bitmap;
            graphics = Graphics.FromImage(bitmap);

            box.MouseDown += OnBoxMouseDown;
        }

        private void OnBoxMouseDown(object sender, MouseEventArgs e)
        {
            int x = e.X;
            int y = e.Y;

            int row = (int)Math.Floor(y / RowH);
            int column = (int)Math.Floor(x / ColumnW);

            F.TryMovePlayerTo(row, column);
        }

        private void OnFieldChanged()
        {
            DrawState();
        }

        private float ColumnW
        {
            get { return boxW / F.Width; }
        }

        private float RowH
        {
            get { return boxH / F.Height; }
        }

        private Field F
        {
            get { return MainManager.Instance.Field; }
        }

        public void DrawState()
        {
            if (graphics == null)
                return;

            graphics.Clear(Color.White);


            if (MainManager.Instance.Field.IsWin)
            {
                graphics.DrawImage(imageWin, 0, 0, boxW, boxH);
                box.Invalidate();
                return;
            }

            if (MainManager.Instance.Field.IsDefeat)
            {
                graphics.DrawImage(imageLoose, 0, 0, boxW, boxH);
                box.Invalidate();
                return;
            }

            foreach (var c in MainManager.Instance.Field.Creatures)
            {
                int row = c.Row;
                int col = c.Column;
                int force = c.Force;

                float textX = col * ColumnW;
                float textY = row * RowH;

                Player p = c as Player;
                if (p != null)
                {

                    Font font = new Font(FontFamily.GenericMonospace, 15, FontStyle.Bold);
                    Brush brush = new SolidBrush(Color.Blue);

                    graphics.DrawImage(imageIvan, textX, textY, ColumnW, RowH);
                    graphics.DrawString(force.ToString(), font, brush, textX, textY);

                    continue;
                }

                Enemy e = c as Enemy;
                if (e != null)
                {
                    if (e.eType == -1)
                        e.eType = MainManager.Instance.Random.Next(0, enemyImages.Length);

                    graphics.DrawImage(enemyImages[e.eType], textX, textY, ColumnW, RowH);

                    Font font = new Font(FontFamily.GenericMonospace, 15, FontStyle.Bold);
                    Brush brush = new SolidBrush(Color.Red);
                    graphics.DrawString(force.ToString(), font, brush, textX, textY);

                    continue;
                }
            }

            for (int i = 1; i < F.Width; i++)
            {
                float xPos = i * ColumnW;
                PointF p1 = new PointF(xPos, 0);
                PointF p2 = new PointF(xPos, boxH);

                graphics.DrawLine(Pens.Black, p1, p2);
            }

            for (int j = 1; j < F.Height; j++)
            {
                float yPos = j * RowH;
                PointF p1 = new PointF(0, yPos);
                PointF p2 = new PointF(boxW, yPos);
                graphics.DrawLine(Pens.Black, p1, p2);
            }

            box.Invalidate();
        }
    }
}
